\name{register}
\alias{register}
\alias{registered}
\alias{bpparam}

\title{Maintain a global registry of available back-end Params}

\description{

  Use functions on this page to add to or query a registry of back-ends,
  including the default for use when no \code{BPPARAM} object is
  provided to functions.

}

\usage{
register(BPPARAM, default=TRUE)
registered(bpparamClass)
bpparam(bpparamClass)
}

\arguments{

  \item{BPPARAM}{
    An instance of a \code{BiocParallelParam} class, e.g.,
    \code{\link{MulticoreParam}}, \code{\link{SnowParam}},
    \code{\link{DoparParam}}.
  }

  \item{default}{
    Make this the default \code{BiocParallelParam} for subsequent
    evaluations? If \code{FALSE}, the argument is placed at the lowest
    priority position.
  }

  \item{bpparamClass}{
    When present, the text name of the \code{BiocParallelParam} class
    (e.g., \dQuote{MulticoreParam}) to be retrieved from the
    registry. When absent, a list of all registered instances is
    returned.
  }
}

\details{

  The registry is a list of back-ends with configuration parameters 
  for parallel evaluation. The first list entry is the default and is 
  used by \code{BiocParallel} functions when no \code{BPPARAM} argument 
  is supplied.

  By default, the registry includes \code{MulticoreParam},
  \code{SnowParam}, \code{BatchJobsParam} and \code{SerialParam}.  The
  objects are constructed from global options of the corresponding name,
  or from the default constructor (e.g., \code{SnowParam()}) if no
  option is specified. The user can set customizations during start-up
  (e.g., in an \code{.Rprofile} file) with, for instance,
  \code{options(MulticoreParam=quote(MulticoreParam(workers=8)))}.

  The act of \dQuote{registering} a back-end modifies the existing
  \code{\link{BiocParallelParam}} in the list; only one param of each 
  type can be present in the registry. When \code{default=TRUE}, the 
  newly registered param is moved to the top of the list thereby making 
  it the default. When \code{default=FALSE}, the param is modified 
  `in place` vs being moved to the top.

  \code{bpparam()}, invoked with no arguments, returns the default 
  \code{\link{BiocParallelParam}} instance from the registry.
  When called with the text name of a \code{bpparamClass}, the 
  global options are consulted first,
  e.g., \code{options(MulticoreParam=MulticoreParam())} and then the
  value of \code{registered(bpparamClass)}.

}

\value{

  \code{register} returns, invisibly, a list of registered back-ends.

  \code{registered} returns the back-end of type \code{bpparamClass} or,
  if \code{bpparamClass} is missing,  a list of all registered back-ends.

  \code{bpparam} returns the back-end of type \code{bpparamClass} or,

}

\author{
  Martin Morgan \url{mailto:mtmorgan@fhcrc.org}.
}

\seealso{

  \code{\link{BiocParallelParam}} for possible values of \code{BPPARAM}.

}

\examples{

## ----------------------------------------------------------------------
## The registry 
## ----------------------------------------------------------------------

## The full registry.
registered()

## When default = TRUE the last param registered becomes the new default.
snowparam <- SnowParam(workers = 3, type = "PSOCK")
register(snowparam, default = TRUE)
registered()

## Retrieve the default back-end,
bpparam()

## or a specific BiocParallelParam.
bpparam("MulticoreParam")

## ----------------------------------------------------------------------
## Specifying a back-end for evaluation
## ----------------------------------------------------------------------

## The back-end of choice is specified in the BPPARAM argument to
## the BiocParallel functions. None, one, or multiple back-ends can be
## provided.

## BPPARAM not specified:
## The default back-end from the registry is used.
bplapply(1:6, sqrt)

## BPPARAM as a single param:
bplapply(1:6, sqrt, BPPARAM = MulticoreParam(3))

## BPPARAM as a list of params:
## Useful for multiple levels of parallel evaluation. The 1:6 are divided
## among Snow workers, each of which dispatch to Multicore workers.
myfun <- function(x, BPPARAM, ...) {
  library(BiocParallel)
  res <- bplapply(x, sqrt, BPPARAM = BPPARAM)
  do.call(mean, res)
}
params <- list(bpparam("SnowParam"), bpparam("MulticoreParam"))
bplapply(list(1:2, 3:4, 5:6), myfun, BPPARAM = params)
}

\keyword{manip}
